/***************************************************************************
                          arrayitem.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Aug 28 1999
    copyright            : (C) 2001 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ARRAYITEM_H
#define ARRAYITEM_H

#include <kurl.h>
#include "kplitem.h"

class KSimpleConfig;

/**
  * Array item class. Class for array items showing data arrays by lines
  * or markers.
  * @author Werner Stille
  */
class ArrayItem : public ScaledItem {

public:
  /** Constructor */
	ArrayItem();
  /** Copy constructor */
  ArrayItem(const ArrayItem& a);
  /**
    * Constructor. Initializes corresponding to autoplot settings.
    * @param aut pointer to structure containing autoplot settings.
    */
	ArrayItem(Kpl::AutoStruct* aut);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param aut pointer to structure containing autoplot settings.
    * @param url URL of plot file.
    */
  ArrayItem(KSimpleConfig* plo, Kpl::AutoStruct* aut, const KURL& url);
  /**
    * Constructor. Initializes corresponding to arguments.
    * @param active true for visible item.
    * @param symb 0 for circles, < 0 for different markers,
    * >0 for different line types.
    * @param color color for function plot.
    * @param fx normalization factor for x values.
    * @param fy normalization factor for y values.
    * @param ix x column index.
    * @param iy y column index.
    * @param ih y error column index.
    * @param istart start row index.
    * @param n number of data points.
    * @param errbars true to plot error bars.
    * @param url URL of data file.
    */
	ArrayItem(bool active, int symb, const QString& color,
            double fx, double fy, int ix, int iy, int ie, int istart,
            int n, int errbars, const KURL& url);
  /** Destructor */
	virtual ~ArrayItem();
  /** Assignment operator */
  ArrayItem& operator=(const ArrayItem& f);
	virtual ItemTypes iType() const;
  virtual void draw(KplGraph* g) const;
  virtual void writePlo(FILE* f, const KURL& url, bool abs) const;
  virtual void setText(KplCheckListItem* it, bool* arrays, bool* funcs) const;
  virtual int editItem(QWidget* parent, KplDoc* m, int i);
  virtual KplItem* copy() const;
  virtual void expoItem(int* iext, int* ieyt, double* fxt, double* fyt) const;
  virtual void minMax(double* xmi, double* xma, double* ymi, double* yma) const;

protected:
  /**
   * Initializes several elements.
   */
  void init();
  /**
   * Reads double values from a file into an array.
   * @return number of lines, 0 for error.
   */
  int readFile();
  /**
   * Deallocates array storage.
   */
  void freeX();
  /**
   * Copies array storage.
   * @return pointer to array.
   */
  double** copyX() const;
  int ix, iy, ie, istart, n, errbars, nrows, ncols;
  double** x;
  KURL url;

  friend class ArrayDlg;
  friend class FitDlg;
  friend class LMFit;
  friend class KplDoc;
  friend class KplApp;
};

#endif
