/***************************************************************************
                          utils.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2001 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <dlfcn.h>
#include <qlineedit.h>
#include <qmultilinedit.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <kglobal.h>
#include <kurl.h>
#include "kpldoc.h"
#include "utils.h"
#include "sourcewidget.h"

ChooseFuncDlg::ChooseFuncDlg(QWidget* parent, FILE* f, QLineEdit* func0) :
 QDialog(parent, 0, true), func(func0)
{
  setCaption(i18n("Functions"));
  resize(200, 200);
  QVBoxLayout* vbox = new QVBoxLayout(this, 11, 6);
  vbox->addWidget(funcList = new QListBox(this));
  char s[256];
  while ((fgets(s, 256, f)))
    if (s[9] == 'T')
      funcList->insertItem(strtok(&s[11], "\n"));
  fclose(f);
  connect(funcList, SIGNAL(selected(int)), SLOT(slotList(int)));
  QHBoxLayout* hbox = new QHBoxLayout(vbox);
  QPushButton* b = new QPushButton(i18n("OK"), this);
  hbox->addWidget(b);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  hbox->addWidget(b = new QPushButton(i18n("Cancel"), this));
  connect(b, SIGNAL(clicked()), SLOT(reject()));
}

ChooseFuncDlg::~ChooseFuncDlg()
{
}

void ChooseFuncDlg::slotList(int i)
{
  func->setText(funcList->text(i));
  accept();
}

void ChooseFuncDlg::slotOK()
{
  int i = funcList->currentItem();
  if (i != -1) {
    func->setText(funcList->text(i));
    accept();
  }
}

EditParameterDlg::EditParameterDlg(QWidget* parent, KplDoc* m, double* p0,
                                   const QString& file, const QString& func) :
 QDialog(parent, 0, true), p(p0)
{
  setCaption(i18n("Parameters"));
  QHBoxLayout* hbox = 0;
  QVBoxLayout* vbox;
  if (m->aut.showSource) {
    hbox = new QHBoxLayout(this, 11, 6);
    vbox = new QVBoxLayout(hbox);
  } else
    vbox = new QVBoxLayout(this, 11, 6);
  QGroupBox* g = new QGroupBox(0, Qt::Vertical, this);
  QGridLayout* grid = new QGridLayout(g->layout(), QMIN(KPL_NPMAX, 10),
                                      2 + 3 * ((KPL_NPMAX - 1) / 10), 6);
  vbox->addWidget(g);
  QString s;
  for (int i = 0; i < KPL_NPMAX; i++) {
    int ix = 3 * (i / 10);
    int iy = i % 10;
    grid->addWidget(new QLabel(s.sprintf("p%i", i), g), iy, ix);
    grid->addWidget(par[i] = new QLineEdit(QString::number(p[i], m->aut.format,
      m->aut.prec), g), iy, ix + 1);
    par[i]->setMaximumWidth(80);
  }
  for (int i = 0; i < ((KPL_NPMAX - 1) / 10); i++)
    grid->addItem(new QSpacerItem(20, 10), 0, 3 * i + 2);
  QHBoxLayout* hbox2 = new QHBoxLayout(vbox);
  QPushButton* b = new QPushButton(i18n("OK"), this);
  hbox2->addWidget(b);
  b->setDefault(true);
  connect(b, SIGNAL(clicked()), SLOT(slotOK()));
  hbox2->addWidget(b = new QPushButton(i18n("Cancel"), this));
  connect(b, SIGNAL(clicked()), SLOT(reject()));
  if (m->aut.showSource) {
    QFileInfo fi(file);
    int pp = func.find("__");
    s = fi.dirPath(true) + "/" + fi.baseName() + ((pp != -1) ? ".cpp" : ".c");
    QFile f(s);
    if (f.open(IO_ReadOnly)) {
      SourceWidget* sw = new SourceWidget(this, &f);
      hbox->addWidget(sw, 1);
      sw->setMinimumWidth(400);
      if (pp != -1) {
        s = func;
        sw->highlight(s.left(pp));
      } else
        sw->highlight(func);
      QTimer::singleShot(0, sw, SLOT(slotHighlight()));
    }
  }
}

EditParameterDlg::~EditParameterDlg()
{
}

void EditParameterDlg::slotOK()
{
  bool valid = true;
  QString s;
  double pt[KPL_NPMAX];
  for (int i = 0; i < KPL_NPMAX; i++) {
    s = par[i]->text();
    pt[i] = s.toDouble(&valid);
    if (!valid) {
      KMessageBox::sorry(this, s.sprintf(i18n("invalid parameter %i"), i));
      return;
    }
  }
  memcpy(p, pt, KPL_NPMAX * sizeof(double));
  accept();
}

bool Utils::getWriteURL(QWidget* parent, KURL& url, const QString& filter,
                        KplDoc* m)
{
  url = KFileDialog::getSaveURL(m->currentDir(), filter);
  if (url.isEmpty())
    return false;
  if (url.isLocalFile()) {
    QFileInfo fi(url.path());
    if (!fi.exists())
      return true;
    if (!fi.isWritable()) {
      KMessageBox::sorry(parent, i18n("Writing to this file not possible."));
      return false;
    }
  } else
    if (!KIO::NetAccess::exists(url))
      return true;
  return (KMessageBox::warningYesNo(parent, url.path() + "\n" +
    i18n("A file with this name exists already.\n"
         "Do you want to overwrite it?")) == KMessageBox::Yes);
}

void Utils::getFile(QLineEdit* e, KplDoc* m)
{
  QString s = KFileDialog::getOpenFileName(m->currentDir(), "*.so\n*");
  if (!s.isEmpty()) {
    e->setText(s);
    m->setCurrentDir(s);
  }
}

void Utils::getFunc(QWidget* parent, QLineEdit* name, QLineEdit* func)
{
  QFileInfo fi(name->text());
  QString path = fi.dirPath(true) + "/" + fi.baseName() + ".def";
  if (FILE* f = fopen(path, "r")) {
    ChooseFuncDlg dlg(parent, f, func);
    dlg.exec();
  } else
    KMessageBox::error(parent,
                       i18n("while trying to open module definition file"));
}

bool Utils::loadPar(const KURL& url, QWidget* parent, double* p, KplDoc* m)
{
  bool success = false;
  QString fn;
  if (KIO::NetAccess::download(url, fn)) {
    if (FILE* f = fopen(fn, "r")) {
      int i;
      for (i = 0; i < KPL_NPMAX; i++)
        p[i] = 0.0;
      i = -1;
      char s[81];
      while (fgets(s, 80, f) && (++i < KPL_NPMAX))
        if (sscanf(s, "%lg", &p[i]) != 1)
          break;
      fclose(f);
      m->setCurrentDir(url);
      success = true;
    } else
      KMessageBox::error(parent,
                         i18n("while trying to open parameter file"));
    KIO::NetAccess::removeTempFile(fn);
  }
  return success;
}

void Utils::getPar(QWidget* parent, double* p, KplDoc* m)
{
  KURL url = KFileDialog::getOpenURL(m->currentDir(), "*.par\n*");
  if (!url.isEmpty())
    loadPar(url, parent, p, m);
}

bool Utils::saveParameter(QWidget* parent, const KURL& url, double* p,
                          double* err, KplDoc* m)
{
  FILE* f;
  if (url.isLocalFile())
    f = fopen(url.path(), "w");
  else
    f = fopen(QDir::homeDirPath() + "/kpltmp.tmp", "w");
  if (f) {
    QCString s1, s2;
    for (int i = 0; i < KPL_NPMAX; i++) {
      s1.setNum(p[i], m->aut.format, m->aut.prec);
      if (err) {
        s2.setNum(err[i], m->aut.format, m->aut.prec);
        fprintf(f, "%s %s\n", s1.data(), s2.data());
      } else
        fprintf(f, "%s\n", s1.data());
    }
    fclose(f);
    m->setCurrentDir(url);
    if (!url.isLocalFile()) {
      m->job = KIO::file_copy(KURL(QDir::homeDirPath() + "/kpltmp.tmp"), url,
                              -1, true);
      m->connectIOfinished();
    }
  } else
    KMessageBox::error(parent, i18n("while trying to open parameter file"));
  return f;
}

void Utils::saveFunPar(QWidget* parent, double* p, KplDoc* m)
{
  KURL url;
  if (getWriteURL(parent, url, "*.par\n*", m))
    saveParameter(parent, url, p, 0, m);
}

bool Utils::getFuncAddr(const QString& path, const QString& name, void** hmod,
                        double (**fkt)(double, const double*))
{
  *hmod = 0;
  *fkt = 0;
  if (path.isEmpty() || name.isEmpty()) {
    KMessageBox::sorry(0, i18n("no library or no function specified!"));
    return true;
  }
  if (!(*hmod = dlopen(path, RTLD_NOW))) {
    KMessageBox::error(0, i18n("while trying to open") + "\n" + path +
                       "\n" + dlerror());
    return true;
  } else {
    *fkt = (double (*)(double, const double *)) dlsym(*hmod, name);
    if (const char* error = dlerror()) {
      KMessageBox::error(0, i18n("while trying to determine the address of") +
                         "\n" + name + "\n" + error);
      dlclose(*hmod);
      *hmod = 0;
      return true;
    } else
      return false;
  }
}

bool Utils::minMaxFile(double* xmin, double* xmax, const double* x, int n)
{
  *xmin = 1.0e300;
  *xmax = -*xmin;
  for (int i = 0; i < n; i++) {
    if ((x[i] > 1.0e300) || (x[i] < -1.0e300))
      return true;
    *xmin = QMIN(*xmin, x[i]);
    *xmax = QMAX(*xmax, x[i]);
  }
  return (*xmax == *xmin);
}

bool Utils::autoSc(double* xmi, double* xma, double* tic, int* mtic, int* ndig,
                   double xmin, double xmax, double fx, double c, bool log)
{
  if (xmax == xmin)
    return true;
  if (log) {
    if (xmin < 1.0e-40)
      return true;
    *ndig = -1;
    double d = c * log10(xmax / xmin);
    if (d > 1.0) {
      *tic = pow(10.0, int(d));
      *mtic = 1;
    } else {
      *tic = 10.0;
      *mtic = 10;
    }
    if (d < 0.7)
      *tic = (d < 0.5) ? 2.0 : 3.0;
    double xl = log10(xmin);
    double mant = xl - int(xl + 1.0e-6 * fabs(xl));
    if (fabs(mant) < 1.0e-9)
      mant = 0.0;
    if (mant < 0.0)
      mant += 1.0;
    int e = qRound(xl - mant);
    if (*tic == 2.0)
      if (mant < log10(1.999999))
        mant = 0.0;
      else
        if (mant < log10(4.999999))
          mant = log10(2.0);
        else
          mant = log10(5.0);
    else
      if (*tic == 3.0)
        if (mant < log10(2.999999))
          mant = 0.0;
        else
          mant = log10(3.0);
      else
        mant = 0.0;
    *xmi = pow(10.0, e + mant);
    xl = log10(xmax);
    mant = xl - int(xl + 1.0e-6 * fabs(xl));
    if (fabs(mant) < 1.0e-9)
      mant = 0.0;
    if (mant < 0.0)
      mant += 1.0;
    e = qRound(xl - mant);
    if (*tic == 2.0)
      if (mant > log10(5.000001))
        mant = 1.0;
      else
        if (mant > log10(2.000001))
          mant = log10(5.0);
        else {
          if (mant)
            mant = log10(2.0);
        }
    else
      if (*tic == 3.0)
        if (mant > log10(3.000001))
          mant = 1.0;
        else {
          if (mant)
            mant = log10(3.0);
        }
      else
        if (mant)
          mant = 1.0;
    *xma = pow(10.0, e + mant);
  } else {
    double d = c * fx * (xmax - xmin);
    double f = pow(10.0, (- (float) (Utils::int9(log10(d + 1.0e-37)))));
    d *= f;
    if (d > 7.071)
      *mtic = 10;
    else
      if (d > 3.162)
        *mtic = 5;
      else
        *mtic = (d > 1.414) ? 2 : 1;
    *tic = *mtic / f;
    if ((*mtic == 1) || (*mtic == 10))
      *mtic = 2;
    if (!(*ndig = -Utils::int9(log10(*tic + 1.0e-37) + 1.0e-6)))
      *ndig = -1;
    *xmi = *tic * Utils::int9(fx * xmin / *tic + 1.0e-4);
    *xma = *tic * Utils::int9(fx * xmax / *tic + 0.9999);
  }
  return false;
}

void Utils::expo(double a, int* ie, double* f)
{
  if ((*ie = int9(log10(1.01 * a + 1.0e-37))) < 0)
    *ie = *ie - 1;
  *ie = -3 * (*ie / 3);
  *f = pow(10.0, *ie);
}

int Utils::int9(double a)
{
  int i = int(a);
  if (a < 0.0)
    i--;
  return i;
}

unsigned Utils::rgbQt(unsigned u)
{
  QColor col(int(u & 0xffL), int((u & 0xff00L) >> 8),
             int((u & 0xff0000L) >> 16));
  return col.rgb();
}

unsigned Utils::rgbQt1(unsigned rgb)
{
  QColor col(rgb);
  return unsigned(col.red()) | (unsigned(col.green()) << 8) |
    (unsigned(col.blue()) << 16);
}

QString Utils::relPath(const KURL& uPlo, const KURL& path, bool abs)
{
  if (!abs) {
    KURL u = path;
    if (!(uPlo.isLocalFile() ^ u.isLocalFile())) {
      if (!uPlo.isLocalFile()) {
        KURL t(uPlo);
        t.setPath("/");
        if (!t.isParentOf(u))
          return path.url();
      }
      QString dPlo = uPlo.directory(false);
      QString dFile = u.directory(false);
      int nMax = QMIN(dPlo.length(), dFile.length());
      int n;
      for (n = 0; n < nMax; n++)
        if (dPlo[n] != dFile[n])
          break;
      if (n > 0)
        n = dFile.findRev('/', n) + 1;
      QString s = u.path().mid(n);
      int nUp = dPlo.contains('/') - dPlo.left(n).contains('/');
      for (int i = 0; i < nUp; i++)
        s.prepend("../");
      if (!u.isLocalFile()) {
        u.setPath(s);
        s = u.url();
      }
      return s;
    }
  }
  return path.isLocalFile() ? path.path() : path.url();
}
